// ADVENT.CPP - Implementation file for your Internet Server
//    'Web Advent' Helper Object

#include "stdafx.h"
#include "Advent.h"
#include "AdventThread.h"  // worker thread needed for processing responses, etc.


///////////////////////////////////////////////////////////////////////
// command-parsing map

BEGIN_PARSE_MAP(CAdventObject, CHttpServer)
	// TODO: insert your ON_PARSE_COMMAND() and 
	// ON_PARSE_COMMAND_PARAMS() here to hook up your commands.
	// For example:

  ON_PARSE_COMMAND(Game, CAdventObject, ITS_PSTR ITS_PSTR)
  ON_PARSE_COMMAND_PARAMS("Command Session")
  ON_PARSE_COMMAND(History, CAdventObject, ITS_PSTR)
  ON_PARSE_COMMAND_PARAMS("Session")
	ON_PARSE_COMMAND(Default, CAdventObject, ITS_EMPTY)
	DEFAULT_PARSE_COMMAND(Default, CAdventObject)
END_PARSE_MAP(CAdventObject)


///////////////////////////////////////////////////////////////////////
// The one and only CAdventObject object

CAdventObject theExtension;


///////////////////////////////////////////////////////////////////////
// CAdventObject implementation

CAdventObject::CAdventObject()
{
}

CAdventObject::~CAdventObject()
{
}

BOOL CAdventObject::GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
	// Call default implementation for initialization
	CHttpServer::GetExtensionVersion(pVer);

	// Load description string
	TCHAR sz[HSE_MAX_EXT_DLL_NAME_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_SERVER, sz, HSE_MAX_EXT_DLL_NAME_LEN));
	_tcscpy(pVer->lpszExtensionDesc, sz);
	return TRUE;
}

BOOL CAdventObject::TerminateExtension(DWORD dwFlags)
{
	// extension is being terminated

	CAdventThread::TerminateAll();

	return TRUE;
}

///////////////////////////////////////////////////////////////////////
// CAdventObject command handlers

void CAdventObject::Game(CHttpServerContext* pCtxt, LPCSTR szCommand, LPCTSTR szSession)
{
	StartContent(pCtxt);
  AddHeader(pCtxt, "pragma: no-cache\r\n");
  AddHeader(pCtxt, "Expires: -1\r\n");

  if(!szSession || !*szSession)
  {
   	WriteTitle(pCtxt);
    *pCtxt << _T("No session!");
    EndContent(pCtxt);
  }
  else
  {
    CString csTemp = CAdventThread::NextWebPage(szCommand, szSession);

    if(!csTemp.GetLength())
    {
     	WriteTitle(pCtxt);
      csTemp = "{ no page }";
      EndContent(pCtxt);
    }

    *pCtxt << csTemp;
  }
}

void CAdventObject::History(CHttpServerContext* pCtxt, LPCTSTR szSession)
{
	StartContent(pCtxt);
  AddHeader(pCtxt, "pragma: no-cache\r\n");
  AddHeader(pCtxt, "Expires: -1\r\n");

// 	WriteTitle(pCtxt);
	*pCtxt << _T("<html><head><title>");
	*pCtxt << _T("On-Line ADVENTURE - snapshot of HISTORY");
	*pCtxt << _T("</title></head><body>");

  if(!szSession || !*szSession)
  {
    *pCtxt << _T("No session!");
  }
  else
  {
    CString csTemp = CAdventThread::GetHistory(szSession);

    if(!csTemp.GetLength())
      csTemp = "{ no history }";
    else
      csTemp = "<PRE>\r\n"
             + csTemp
             + "\r\n</PRE>\r\n";

    *pCtxt << csTemp;
  }

  EndContent(pCtxt);
}

void CAdventObject::Default(CHttpServerContext* pCtxt)
{
	StartContent(pCtxt);
  AddHeader(pCtxt, "pragma: no-cache\r\n");
  AddHeader(pCtxt, "Expires: -1\r\n");

  *pCtxt << CAdventThread::InitialWebPage();

}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CAdventObject, CHttpServer)
	//{{AFX_MSG_MAP(CAdventObject)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0



///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/

// THESE were copied/adapted from the base class so I can see what they do

LPCTSTR CAdventObject::GetTitle() const
{
	return(_T("On-Line ADVENTURE for IIS (660 point version)"));
}

void CAdventObject::StartContent(CHttpServerContext* pCtxt) const
{
//	CHttpServer::StartContent(pCtxt);

	AddHeader(pCtxt, _T("Content-Type: text/html\r\n"));
}

void CAdventObject::EndContent(CHttpServerContext* pCtxt) const
{
//	CHttpServer::EndContent(pCtxt);

  *pCtxt << _T("</body></html>");
}

void CAdventObject::WriteTitle(CHttpServerContext* pCtxt) const
{
//	CHttpServer::WriteTitle(pCtxt);

	*pCtxt << _T("<html><head><title>");
	*pCtxt << GetTitle();
	*pCtxt << _T("</title></head><BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\">");
}
