// AdventThread.cpp : implementation file
//

#include "stdafx.h"
#include "Advent.h"
#include "AdventThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NHIST_LINES 20 /* # of lines of history visible on regular page */

/////////////////////////////////////////////////////////////////////////////
// CAdventThread

CMutex CAdventThread::m_Mutex;  // global mutex for the thread mapper thingy

CArray<CAdventThread *,CAdventThread *> CAdventThread::m_aThreads;  // thread mapping - string ID to CWinThread object

extern "C" int __cdecl AdventThreadEntry( void * pParam); // main thread entry (in 'advkern.c')


IMPLEMENT_DYNCREATE(CAdventThread, CWinThread)

CAdventThread::CAdventThread()
{
  m_dwStartTick = GetTickCount();  // important for timeouts
  m_bExitFlag = FALSE;
  m_pGlobals = NULL;

  m_lInInputFlag = 0;

  CSingleLock lock(&m_Mutex, TRUE);

  void *pThread = NULL;

  do
  {
    m_csSession.Format("%08X%08X", GetTickCount(), ((DWORD)this ^ rand()));

    Sleep(1);  // just in case

    for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
    {
      if(!m_aThreads[i1] || IsBadReadPtr(m_aThreads[i1], sizeof(CAdventThread)))
      {
        m_aThreads.RemoveAt(i1);  // housecleaning
        continue;
      }

      if(!m_csSession.CompareNoCase(m_aThreads[i1]->m_csSession))
      {
        m_csSession = ""; // as a loop flag
        break;
      }
    }

  } while(!m_csSession.GetLength());  // assuming I never find it anyway

  m_aThreads.Add(this);

  lock.Unlock();


  // TODO:  additional thingies
}

CAdventThread::~CAdventThread()
{
  CSingleLock lock(&m_Mutex, TRUE);

  for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
  {
    if(m_aThreads[i1] == this)
      m_aThreads.RemoveAt(i1);
  }

  lock.Unlock();


  // do other things now

  if(m_pGlobals)
  {
    GlobalFree(GlobalHandle(m_pGlobals));
    m_pGlobals = NULL;
  }
}

void CAdventThread::TerminateAll()
{
  CSingleLock lock(&m_Mutex, TRUE);

  for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
  {
    if(!m_aThreads[i1] || IsBadReadPtr(m_aThreads[i1], sizeof(CAdventThread)))
    {
      m_aThreads.RemoveAt(i1);  // housecleaning
      continue;
    }

    m_aThreads[i1]->m_bExitFlag = TRUE;
    m_aThreads[i1]->m_csOutQueue = "";  // another flag to exit *NOW*
  }


  do
  {
    lock.Unlock();

    Sleep(50);

    lock.Lock();

    for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
    {
      if(!m_aThreads[i1] || IsBadReadPtr(m_aThreads[i1], sizeof(CAdventThread)))
      {
        m_aThreads.RemoveAt(i1);  // housecleaning
        continue;
      }

      m_aThreads[i1]->m_bExitFlag = TRUE;
      m_aThreads[i1]->m_csOutQueue = "";  // another flag to exit *NOW*
    }

  } while(m_aThreads.GetSize());

}


CString CAdventThread::InitialWebPage()
{
static const char szWelcome1[] = 
"<HTML>\r\n"
"<HEAD>\r\n"
"<META NAME=\"GENERATOR\" Content=\"Adventure ISAPI DLL\">\r\n"
"<META HTTP-EQUIP=\"pragma\" CONTENT=\"no-cache\">\r\n"
"<META HTTP-EQUIP=\"Expires\" CONTENT=\"-1\">\r\n"
"<TITLE>ADVENTURE - THE ON-LINE VERSION</TITLE>\r\n"
"</HEAD>\r\n"
"<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" >\r\n"
"    <CENTER>\r\n"
"    <H1>ADVENTURE - the On-Line version</H1>\r\n"
"    </CENTER>\r\n"
"    <p>\r\n"
"    Hi.  Welcome to the HTML version of the RJR Cave 'Adventure' program,\r\n"
"    one of the first (if not <strong><i>THE</i></strong> first text-based\r\n"
"    adventure.  The version that you see here is based on the 660 point\r\n"
"    1991 version that's 'copyleft' by M.L.Arnautov, and the source files\r\n"
"    were originally downloaded from the following web site:<br>\r\n"
"    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"http://www.rickadams.org/adventure/\">\r\n"
"    http://www.rickadams.org/adventure/</a><br>\r\n"
"    That being said, if you want to play ADVENTURE, press the 'OK' button below.\r\n"
"    </p>\r\n"
"    <INPUT TYPE=SUBMIT VALUE=\"OK\" LANGUAGE=\"JScript\" \r\n"
"     OnClick=\"window.navigate('advent.dll?Game?Command=%20&Session=";
// insert session string here

static const char szWelcome2[] = 
"');\">\r\n"
"<P>&nbsp;</P>\r\n"
"<p>\r\n"
"NOTE:  for hints on playing the game, go \r\n"
"<a href=\"http://www.rickadams.org/adventure/d_hints/index.html\">HERE</a>\r\n"
"</p>"
"</BODY>\r\n"
"</HTML>\r\n";

  CAdventThread *pThread = (CAdventThread *)AfxBeginThread(RUNTIME_CLASS(CAdventThread),
                                                           THREAD_PRIORITY_NORMAL,
                                                           0, CREATE_SUSPENDED, NULL);
  if(!pThread)
    return("");  // blank indicates an error.

  CString csRval = szWelcome1 + pThread->m_csSession + szWelcome2;

  pThread->m_csLastPage = csRval;

  pThread->ResumeThread();

  return(csRval);  // the web page that I return - yay!
}

CString CAdventThread::NextWebPage(LPCSTR szCommand, LPCSTR szSession)
{
static const char szNoThread[] = 
"<HTML>\r\n"
"<HEAD>\r\n"
"<META NAME=\"GENERATOR\" Content=\"Adventure ISAPI DLL\">\r\n"
"<META HTTP-EQUIP=\"pragma\" CONTENT=\"no-cache\">\r\n"
"<META HTTP-EQUIP=\"Expires\" CONTENT=\"-1\">\r\n"
"<TITLE>ADVENTURE - THE ON-LINE VERSION</TITLE>\r\n"
"</HEAD>\r\n"
"<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" >\r\n"
"    <CENTER>\r\n"
"    <H1>ADVENTURE - SESSION ENDED</H1>\r\n"
"    </CENTER>\r\n"
"    <p>\r\n"
"    If you want to start ADVENTURE again, press the 'OK' button below.\r\n"
"    </p>\r\n"
"    <INPUT TYPE=SUBMIT VALUE=\"OK\" LANGUAGE=\"JScript\" \r\n"
"     OnClick=\"window.navigate('advent.dll');\">\r\n"
"<P>&nbsp;</P>\r\n"
"</BODY>\r\n"
"</HTML>\r\n";

static const char szPageTemplate1[] =
"<HTML>\r\n"
"<HEAD>\r\n"
"<META NAME=\"GENERATOR\" Content=\"Adventure ISAPI DLL\">\r\n"
"<META HTTP-EQUIP=\"pragma\" CONTENT=\"no-cache\">\r\n"
"<META HTTP-EQUIP=\"Expires\" CONTENT=\"-1\">\r\n"
"<TITLE>ADVENTURE - THE ON-LINE VERSION</TITLE>\r\n"
"</HEAD>\r\n"
"<BODY BGCOLOR=\"#ffffff\" TEXT=\"#000000\" LANGUAGE=\"JScript\" OnLoad=\"focus(); form1.focus();\">\r\n"
"    <CENTER>\r\n"
"    <H1>ADVENTURE - the On-Line version</H1>\r\n"
"    </CENTER>\r\n";
static const char szPageTemplate_separator_start[] =
"    <font size=\"-1\" color=\"#808080\"><PRE WRAP=\"OFF\">\r\n";

// insert response text here

static const char szPageTemplate_separator_end [] = 
"    </PRE></font>\r\n";

static const char szPageTemplate_begin_output [] =
"    <PRE WRAP=\"OFF\">\r\n";

// this goes between the 'history' portion and the "live" part

static const char szPageTemplate2[] =
"    </PRE>\r\n"
"    <FORM LANGUAGE=\"JScript\" METHOD=GET ACTION=\"\"\r\n"
"     onSubmit=\"window.navigate('advent.dll?Game?Command='+Command.value+'&Session=";

// insert session string here

static const char szPageTemplate3[] =
"'); return false;\"\r\n"
"     onFocus=\"Command.focus();\"\r\n"
"     id=form1 name=form1>\r\n"
"    <INPUT TYPE=SUBMIT VALUE=\"OK\" id=Game>\r\n"
"    <INPUT TYPE=TEXT NAME=\"Command\" VALUE=\"\" id=Command size=72>\r\n"
"    </FORM>\r\n"
"<P>&nbsp;</P>\r\n"
"    <p>\r\n"
"    <INPUT TYPE=BUTTON VALUE=\"Show HISTORY\" LANGUAGE=JScript OnClick=\"window.showModelessDialog('advent.dll?History?Session=";

// insert session string here

static const char szPageTemplate4[] =
"');\">\r\n"
"    </p>\r\n"
"<P>&nbsp;</P>\r\n"
"<!-- the 'all alike' maze goes like this:\r\n"
"e, s, s, s, n, e (orange column is here), e, and nw. treasure chest is here\r\n"
"To escape, go se, n, and d. You will find yourself at the bird chamber room.\r\n"
"-->\r\n"
"</BODY>\r\n"
"</HTML>\r\n";

  CSingleLock lock(&m_Mutex, FALSE);
  CString csTemp, csCommand = szCommand;

  csCommand.TrimLeft();
  csCommand.TrimRight();

  lock.Lock();

  CAdventThread *pThread = ThreadFromIdentifier(szSession);;
  DWORD dwTick = GetTickCount();
  
  while(pThread)    // wait until the command buffer is empty
  {
    if(!pThread->m_csInQueue.GetLength())
      break;

    if((GetTickCount() - pThread->m_dwStartTick) > 300000 ||
       (GetTickCount() - dwTick) > 15000)  // wait up to 15 seconds
    {
      break;
    }

    lock.Unlock();

    Sleep(50);

    lock.Lock();

    pThread = ThreadFromIdentifier(szSession);
  }

  if(pThread)
  {
    while(pThread->m_acsHistory.GetSize() > 126)
    {
      pThread->m_acsHistory.RemoveAt(0);
    }

    if(csCommand.GetLength())
    {
      pThread->m_dwStartTick = dwTick = GetTickCount(); // start timeout timer over

      pThread->m_acsHistory.Add(szCommand);
      pThread->m_csInQueue = csCommand;  // overwrite previous (just because)

      pThread->m_Event.SetEvent();  // should make things go very fast

      do // wait until the command buffer is empty
      {
        lock.Unlock();

        Sleep(50);

        lock.Lock();

        pThread = ThreadFromIdentifier(szSession);

        if(!pThread || !pThread->m_csInQueue.GetLength())
          break;

      } while((GetTickCount() - dwTick) <= 15000);  // wait up to 15 seconds
    }
  }

  if(pThread)
  {
    if(!csCommand.GetLength() && !pThread->m_csOutQueue.GetLength())
    {
      csTemp = pThread->m_csLastPage;
    }
    else 
    {
      // wait up to 15 seconds for the output queue to end in a '?' or
      // for the thread to be marked 'quit'

      do // wait until the displayed string ends in a '?'
      {
        lock.Unlock();

        Sleep(50);

        lock.Lock();

        pThread = ThreadFromIdentifier(szSession);

        if(!pThread || pThread->m_bExitFlag)
          break;

        if(pThread->m_lInInputFlag)  // I am currently waiting for input
        {
          int iLen = pThread->m_csOutQueue.GetLength();

          if(iLen > 0 &&
             (pThread->m_csOutQueue[iLen - 1] == '?' ||
              pThread->m_csOutQueue[iLen - 1] == ':'))
          {
            if(iLen > 1 &&
               pThread->m_csOutQueue[iLen - 1] == '?' &&
               pThread->m_csOutQueue[iLen - 2] == '\n')
            {
              // trim off an "<LF>?"

              pThread->m_csOutQueue = pThread->m_csOutQueue.Left(iLen - 2);
            }
          }
          else if(iLen > 1 &&
                  (pThread->m_csOutQueue[iLen - 2] == '?' ||
                   pThread->m_csOutQueue[iLen - 2] == ':') &&
                   pThread->m_csOutQueue[iLen - 1] <= ' ')
          {
            if(iLen > 2 &&
               pThread->m_csOutQueue[iLen - 2] == '?' &&
               pThread->m_csOutQueue[iLen - 3] == '\n')
            {
              // trim off an "<LF>? "

              pThread->m_csOutQueue = pThread->m_csOutQueue.Left(iLen - 3);
            }

            break;
          }
          else if(iLen > 2 &&
                  (pThread->m_csOutQueue[iLen - 3] == '?' ||
                   pThread->m_csOutQueue[iLen - 3] == ':') &&
                  pThread->m_csOutQueue[iLen - 2] <= ' ' &&
                  pThread->m_csOutQueue[iLen - 1] <= ' ')
          {
            if(iLen > 3 &&
               pThread->m_csOutQueue[iLen - 3] == '?' &&
               pThread->m_csOutQueue[iLen - 4] == '\n')
            {
              // trim off an "<LF>?  "

              pThread->m_csOutQueue = pThread->m_csOutQueue.Left(iLen - 4);
            }
          }

          break;
        }

      } while((GetTickCount() - dwTick) <= 15000);  // wait up to 15 seconds

      if(pThread)
      {
        // strip off any leading LF or CRLF combination.

        if(pThread->m_csOutQueue.GetLength() >= 2 &&
           !strncmp((LPCSTR)pThread->m_csOutQueue, "\r\n", 2))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Mid(2);
        }

        if(pThread->m_csOutQueue.GetLength() >= 1 && pThread->m_csOutQueue[0] == '\n')
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Mid(1);
        }

        // strip off any terminating LF or CRLF combination.  4 TIMES.

        if(pThread->m_csOutQueue.GetLength() >= 2 &&
           !strcmp((LPCSTR)pThread->m_csOutQueue + pThread->m_csOutQueue.GetLength() - 2, "\r\n"))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 2);
        }
        else if(pThread->m_csOutQueue.GetLength() >= 1 &&
                (pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\n' ||
                 pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\r'))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 1);
        }

        if(pThread->m_csOutQueue.GetLength() >= 2 &&
           !strcmp((LPCSTR)pThread->m_csOutQueue + pThread->m_csOutQueue.GetLength() - 2, "\r\n"))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 2);
        }
        else if(pThread->m_csOutQueue.GetLength() >= 1 &&
                (pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\n' ||
                 pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\r'))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 1);
        }

        if(pThread->m_csOutQueue.GetLength() >= 2 &&
           !strcmp((LPCSTR)pThread->m_csOutQueue + pThread->m_csOutQueue.GetLength() - 2, "\r\n"))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 2);
        }
        else if(pThread->m_csOutQueue.GetLength() >= 1 &&
                (pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\n' ||
                 pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\r'))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 1);
        }

        if(pThread->m_csOutQueue.GetLength() >= 2 &&
           !strcmp((LPCSTR)pThread->m_csOutQueue + pThread->m_csOutQueue.GetLength() - 2, "\r\n"))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 2);
        }
        else if(pThread->m_csOutQueue.GetLength() >= 1 &&
                (pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\n' ||
                 pThread->m_csOutQueue[pThread->m_csOutQueue.GetLength() - 1] == '\r'))
        {
          pThread->m_csOutQueue = pThread->m_csOutQueue.Left(pThread->m_csOutQueue.GetLength() - 1);
        }

        if(!pThread->m_csOutQueue.GetLength() &&
           pThread->m_csLastPage.GetLength())  // consider blank lines to be ignorable
        {
          csTemp = pThread->m_csLastPage;
        }
        else
        {
          // grab last "up to NHIST_LINES" lines from history

          csTemp = "";

          int i1, i2, i3;
          for(i1=pThread->m_acsHistory.GetSize() - 1, i2=0; i1 >= 0 && i2 <= NHIST_LINES; i1--)
          {
            // count LF's in line, add to 'i2'

            i3 = 0;
            while((i3 = pThread->m_acsHistory[i1].Find('\n', i3)) >= 0)
            {
              i3++;
              i2++;
            }

            i2++;  // and one more for the line I am currently working on


            if(i2 <= NHIST_LINES)
            {
              if(!csTemp.GetLength())
                csTemp = pThread->m_acsHistory[i1];  // no CRLF after it
              else
                csTemp = pThread->m_acsHistory[i1]
                       + "\r\n"
                       + csTemp;
            }
          }

          if(csTemp.GetLength())
            csTemp = szPageTemplate_separator_start
                   + csTemp
                   + szPageTemplate_separator_end;

          csTemp = szPageTemplate1
                 + csTemp
                 + szPageTemplate_begin_output
                 + pThread->m_csOutQueue  // should not end in CRLF even
                 + szPageTemplate2
                 + pThread->m_csSession
                 + szPageTemplate3
                 + pThread->m_csSession
                 + szPageTemplate4;

          pThread->m_csLastPage = csTemp;
          if(pThread->m_csOutQueue.GetLength())
          {
            pThread->m_acsHistory.Add(pThread->m_csOutQueue);
            pThread->m_csOutQueue = "";
          }
        }
      }
    }
  }
  else
  {
#ifdef _DEBUG
    csTemp = (CString)"{ NO THREAD - szSession = '"
           + (CString)szSession
           + (CString)"' }";

    for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
    {
      if(!m_aThreads[i1] || IsBadReadPtr(m_aThreads[i1], sizeof(CAdventThread)))
      {
        m_aThreads.RemoveAt(i1);  // housecleaning
        continue;
      }

      CString csKey = m_aThreads[i1]->m_csSession;

      csTemp += "\r\nSession " + csKey;

      if(csKey.CompareNoCase(pThread->m_csSession))
        csTemp += " !! ";
      else
        csTemp += "    ";

      csKey.Format("at %08xH", (DWORD)m_aThreads[i1]);
      csTemp += csKey;
    }
    csTemp += "\r\n";
#else 

    csTemp = szNoThread;  // just send THAT back

#endif // _DEBUG
  }

  lock.Unlock();

  return(csTemp);
}

CString CAdventThread::GetHistory(LPCSTR szSession)
{
  CString csTemp;

  CSingleLock lock(&m_Mutex, TRUE);

  CAdventThread *pThread = ThreadFromIdentifier(szSession);

  if(pThread)
  {
    int i1;
    for(i1=0; i1 < pThread->m_acsHistory.GetSize(); i1++)
    {
      csTemp += pThread->m_acsHistory[i1] + "\r\n";
    }

    if(!csTemp.GetLength())
      csTemp = "{ NO HISTORY }";
  }
  else
  {
    csTemp = (CString)"{ NO THREAD - szSession = '"
#ifdef _DEBUG
           + (CString)szSession
#endif // _DEBUG
           + (CString)"' }";

#ifdef _DEBUG
    for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
    {
      if(!m_aThreads[i1] || IsBadReadPtr(m_aThreads[i1], sizeof(CAdventThread)))
      {
        m_aThreads.RemoveAt(i1);  // housecleaning
        continue;
      }

      CString csKey = m_aThreads[i1]->m_csSession;

      csTemp += "\r\nSession " + csKey;

      if(csKey.CompareNoCase(pThread->m_csSession))
        csTemp += " !! ";
      else
        csTemp += "    ";

      csKey.Format("at %08xH", (DWORD)m_aThreads[i1]);
      csTemp += csKey;
    }
#endif // _DEBUG
    csTemp += "\r\n";
  }

  lock.Unlock();

  return(csTemp);
}


CAdventThread *CAdventThread::ThreadFromIdentifier(LPCSTR szIdentifier)
{
  CAdventThread *pThread = NULL;
  
  for(int i1 = m_aThreads.GetSize() - 1; i1 >= 0; i1--)
  {
    if(!m_aThreads[i1] || IsBadReadPtr(m_aThreads[i1], sizeof(CAdventThread)))
    {
      m_aThreads.RemoveAt(i1);  // housecleaning
      continue;
    }

    if(!m_aThreads[i1]->m_csSession.CompareNoCase(szIdentifier))
    {
      pThread = m_aThreads[i1];
      break;
    }
  }

  if(pThread && !pThread->IsKindOf(RUNTIME_CLASS(CAdventThread)))
    pThread = NULL;

  return(pThread);  // at this point the thread may *NOT* be 'valid'
}

BOOL CAdventThread::InitInstance()
{
  static CWnd wndNULL;
  m_pMainWnd = &wndNULL;

  m_dwStartTick = GetTickCount();

  m_bExitFlag = FALSE;

#ifdef _DEBUG
  SafeWriteOutput("Adventure starting up...\r\n\r\n");
#endif // _DEBUG

  int iRval = AdventThreadEntry(NULL);

#ifdef _DEBUG
  if(iRval > 0)
    SafeWriteOutput("AdventThreadEntry exits with positive return value\r\n");
  else if(iRval < 0)
    SafeWriteOutput("AdventThreadEntry exits with negative return value\r\n");
  else
    SafeWriteOutput("AdventThreadEntry exits with zero return value\r\n");
#endif // _DEBUG
  
  m_bExitFlag = TRUE;

  // wait for output queue to empty, or for 5 minutes (whichever is longer)

  m_dwStartTick = GetTickCount();

  CSingleLock lock(&m_Mutex, TRUE);

  while(m_csOutQueue.GetLength() &&
        (GetTickCount() - m_dwStartTick) < 300000)  // 5 minutes
  {
    lock.Unlock();

    Sleep(500);

    lock.Lock();
  }

  lock.Unlock();


  // ANYTHING ELSE???


  return(FALSE);
}

int CAdventThread::ExitInstance()
{
  m_pMainWnd = NULL;  // make damn sure

  // additional cleanup goes here

	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(CAdventThread, CWinThread)
	//{{AFX_MSG_MAP(CAdventThread)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CAdventThread utilties

int CAdventThread::SafeWriteOutput(LPCSTR szOut)
{
  CSingleLock lock0(&m_Mutex, TRUE);  // wait infinitely for this, to begin with

  int i1;
  for(i1=0; szOut[i1]; i1++)
  {
    char cOut = szOut[i1];  // character conversions - heh...

    if(cOut == '\n')
      m_csOutQueue += "\r\n";
    else if(cOut == '<')
      m_csOutQueue += "&lt;";
    else if(cOut == '>')
      m_csOutQueue += "&gt;";
    else if(cOut == '&')
      m_csOutQueue += "&amp;";
    else // if(cOut)
      m_csOutQueue += cOut;
  }

  lock0.Unlock();  // by convention

  return(strlen(szOut));
}

void CAdventThread::SafeWriteOutput(char cOut)
{
  CSingleLock lock0(&m_Mutex, TRUE);  // wait infinitely for this, to begin with

  if(cOut == '\n')
    m_csOutQueue += "\r\n";
  else if(cOut == '<')
    m_csOutQueue += "&lt;";
  else if(cOut == '>')
    m_csOutQueue += "&gt;";
  else if(cOut == '&')
    m_csOutQueue += "&amp;";
  else if(cOut)
    m_csOutQueue += cOut;

  lock0.Unlock();  // by convention
}

CString CAdventThread::SafeReadInput()  // reads a (well, THE) line of text
{
  CInInput in_input(m_lInInputFlag);  // this is my auto-flag for "in input"

  CString csRval;

  if(m_bExitFlag)
  {
    return("");
  }
  else  // the 'else' helps me scope the next part
  {
    // scope this - must!

    CSingleLock lock0(&m_Mutex, TRUE);  // wait infinitely for this, to begin with

    if(m_csInQueue.GetLength())
    {
      csRval = m_csInQueue;
      m_dwStartTick = GetTickCount();  // reset this *NOW*

      m_csInQueue = "";

      lock0.Unlock();  // by convention

      return(csRval);
    }

    lock0.Unlock();  // by convention (make sure)
  }

  CSyncObject *aSync[2];

  aSync[0] = &m_Event;
  aSync[1] = &m_Mutex;

  CMultiLock lock(aSync, sizeof(aSync)/sizeof(aSync[0]), FALSE);

  while(!m_bExitFlag)
  {
    DWORD dwLock = lock.Lock(1000, TRUE, 0);

    if(dwLock >= WAIT_OBJECT_0 && 
       dwLock < (WAIT_OBJECT_0 + sizeof(aSync)/sizeof(aSync[0])))
    {
      lock.Unlock();  // assume it might be locked

      if(m_csInQueue.GetLength())
      {
        csRval = m_csInQueue;

        m_csInQueue = "";

        lock.Unlock();  // by convention unlock them all

        return(csRval);
      }
    }
    else
    {
      Sleep(50);
    }

    if(lock.IsLocked(0) || lock.IsLocked(1))
    {
      if(lock.IsLocked(1))  // mutex was locked ?  just in case
      {
        if(m_csInQueue.GetLength())
        {
          csRval = m_csInQueue;

          m_csInQueue = "";

          lock.Unlock();  // by convention unlock them all

          return(csRval);
        }
      }

      lock.Unlock();  // make sure
    }

    if((GetTickCount() - m_dwStartTick) >= 300000) // 5 minute timeout
    {
      m_bExitFlag = TRUE;
      break;
    }
  }

  return("");  // an error
}







/////////////////////////////////////////////////////////////////////////////
// CAdventThread message handlers





// UTILITIES NEEDED TO MAKE THIS WORK

#include "Adventure4\advkern.h"

extern int __cdecl AdventThreadEntry(void * pParam); // main thread entry ('advkern.c')

extern "C" int __cdecl WriteS(const char *szString)
{
  CAdventThread *pThread = (CAdventThread *)AfxGetThread();

  if(!pThread || !pThread->IsKindOf(RUNTIME_CLASS(CAdventThread)))
    return(0);

  return(pThread->SafeWriteOutput(szString));
}

extern "C" int __cdecl WriteF(const char *szFmt, ...)
{
  va_list __alist;
  va_start(__alist, szFmt);

  CString csTemp;
  csTemp.FormatV(szFmt, __alist);

  va_end(__alist);

  return(WriteS(csTemp));
}

extern "C" int __cdecl WriteC(char cData)
{
  CAdventThread *pThread = (CAdventThread *)AfxGetThread();

  if(!pThread || !pThread->IsKindOf(RUNTIME_CLASS(CAdventThread)))
    return(0);

  pThread->SafeWriteOutput(cData);

  return(1);
}

extern "C" int __cdecl ReadS(char *szBuf, int nMaxChars)
{
  CAdventThread *pThread = (CAdventThread *)AfxGetThread();

  CString csTemp;

  if(!pThread || !pThread->IsKindOf(RUNTIME_CLASS(CAdventThread)))
  {
    csTemp = "Y";  // force it to become 'YES'
  }
  else
  {
    csTemp = pThread->SafeReadInput();

    if(!csTemp.GetLength())  // indicating 'shutdown'
      csTemp = "Y";          // force it to become 'YES'
  }

  csTemp += '\n';

  memset(szBuf, 0, nMaxChars);
  memcpy(szBuf, csTemp, min(csTemp.GetLength(), nMaxChars - 1));

  return(strlen(szBuf));  // returns # of characters actually "read"
}

extern "C" int __cdecl MustQuit()
{
  CAdventThread *pThread = (CAdventThread *)AfxGetThread();

  if(!pThread || !pThread->IsKindOf(RUNTIME_CLASS(CAdventThread)))
  {
    return(-1);
  }
  else
  {
    return(pThread->GetExitFlag());
  }
}

extern "C" _GLOBAL_1_ * __cdecl GetGlobalPointer1()
{
  CAdventThread *pThread = (CAdventThread *)AfxGetThread();

  if(!pThread || !pThread->IsKindOf(RUNTIME_CLASS(CAdventThread)))
  {
    return(NULL);  // startup proc must be able to check for this
  }

  _GLOBAL_1_ *pVars = (_GLOBAL_1_ *)pThread->m_pGlobals;

  if(!pThread->m_pGlobals)
  {
    pThread->m_pGlobals = (BYTE *)GlobalLock(GlobalAlloc(GHND, sizeof(_GLOBAL_1_) + 16));

    if(!pThread->m_pGlobals)
      return(NULL);  // still an error

    pVars = (_GLOBAL_1_ *)pThread->m_pGlobals;

    memset(pVars, 0, sizeof(_GLOBAL_1_));  // just in case

    pVars->__u0    = NULL;

    pVars->__c1[0] = '\n';
    pVars->__h1[0] = '\n';
    pVars->__e2    = 0;
    pVars->__k2    = pVars->__p1;
    pVars->__t2    = -1;

    pVars->__g2    = 1;
  }

  return(pVars);
}

